/*
	libpowermate v1.0
	Griffin PowerMate handling library
	Copyright(C) 2004 Manuel Sainz de Baranda Goñi
	Copyright(C) 2004 The Free Software Foundation
	Distributed under the terms of the GNU Public License v2
*/

#include "powermate.h"
#include <linux/input.h>
#include <stddef.h>
#include <errno.h>
#include <fcntl.h>
#include <dirent.h>
#include <stdio.h>


struct pm_event {
	long a;
	long b;
	short type;
	short code;
	unsigned int data;
};


char *check_powermate_device(int fd)
	{
	unsigned int index;
	char id_buff[256];
	char *id_strings[] = {
		"Griffin SoundKnob",
		"Griffin PowerMate"
	};

	ioctl(fd, EVIOCGNAME(255), &id_buff);
	for (index = 0; index != 2; index ++)
		if (!strcmp(id_strings[index], id_buff)) return id_strings[index];
	return NULL;
	}


unsigned char check_powermate_support(void)
	{
	}


int search_powermate_devices(char *directory, ***found_devices)
	{
	int fd, count = 0, size;
	DIR* dir;
	char default_directory[] = "/dev/input", *name, **devices;
	union {	struct dirent entry;
              	char b[offsetof(struct dirent, d_name) + NAME_MAX + 1];
	} u;
	struct dirent *result = &u.entry;

	if (directory == NULL) directory = default_directory;
	if ((dir = opendir(directory)) == NULL) return NULL;

	while (result == &u.entry)
		{
		if (readdir_r(dir, &u.entry, &result) < 0) break;
		if ((fd = open(u.entry.d_name, O_RDONLY)) == -1
		|| check_powermate_device(fd) == NULL)
			continue;

		name = malloc(size = strlen(u.entry.d_name));
		memcpy(name, u.entry.d_name, size);
		count++;
		}

	}


PowerMate *powermate_new(char *device, PowerMateHandlers *handlers)
	{
	PowerMate *pm = (PowerMate *)malloc(sizeof(PowerMate));

	if (pm == NULL)
		{
		errno = ENOMEM;
		return NULL;
		}

	if (stat(device, &pm->device_stat))
		goto failed;
	if (!S_ISCHR(pm->device_stat.st_mode))
		goto failed_no_device;
	if ((pm->input = open(device, O_RDONLY)) == -1)
		goto failed;
	if ((pm->model_id = check_powermate_device(pm->input)) == NULL)
		goto failed_no_device;
	pm->output = open(device, O_WRONLY);
	if (handlers != NULL) powermate_set_handlers(pm, handlers);

	return pm;

	failed_no_device:
		errno = ENODEV;
	failed:
		free(pm);
		return NULL;
	}


void powermate_get_events(PowerMate *pm)
	{
	struct pm_event event;

	while (read(pm->input, &event, sizeof(struct pm_event)) != -1)
		{
		switch (event.type)
			{
			case EV_KEY:
				if (event.data == 0)
					{
					if (pm->handlers.on_up != NULL)
						pm->handlers.on_up(pm, NULL);
					}

				else if (event.data == 1)
					{
					if (pm->handlers.on_down != NULL)
						pm->handlers.on_down(pm, NULL);
					}
				break;

			case EV_REL:
				if ((int)event.data > 0)
					{
					if (pm->handlers.on_right != NULL)
						pm->handlers.on_right(pm, NULL, event.data);
					}

				else if ((int)event.data < 0)
					{
					if (pm->handlers.on_left != NULL)
						pm->handlers.on_left(pm, NULL, (unsigned int)-(int)event.data);
					}

				else printf("ZERO\n");

				break;

			case EV_MSC:
				pm->led.static_brightness	= (unsigned char)(event.data & 0xFF);
				pm->led.pulse_speed		= (unsigned short)(event.data >> 8) & 0x1FF;
				pm->led.pulse_table		= (unsigned char)(event.data >> 17) & 3;
				pm->led.pulse_asleep		= (unsigned char)(event.data >> 19) & 1;
				pm->led.pulse_awake		= (unsigned char)(event.data >> 20) & 1;
				break;
			}
		}

	pm->input = -1;
	}


int powermate_set_led(PowerMate *pm, PowerMateLED *led)
	{
	if (led == NULL) led = &pm->led;

	if (pm->output < 0)
		{
		errno = EBADF;
		return -1;
		}

	if (led->pulse_table > 2 || led->pulse_asleep > 1 || led->pulse_awake > 1)
		{
		errno = EINVAL;
		return -1;
		}

	else	{
		struct pm_event e =
			{
			0, 0, EV_MSC, MSC_PULSELED,
			((unsigned int)led->static_brightness & 0xFF)
			| ((unsigned int)(led->pulse_speed & 0x1FF) << 8)
			| ((unsigned int)(led->pulse_table) << 17)
			| ((unsigned int)(led->pulse_asleep) << 19)
			| ((unsigned int)(led->pulse_awake) << 20)
			};

		if (write(pm->output, &e, sizeof(struct pm_event)) < 0) return -1;
		if (led != &pm->led)
			memmove(&pm->led, led, sizeof(PowerMateLED));
		return 0;
		}
	}

int powermate_set_static_brightness(PowerMate *pm, unsigned char brightness)
	{
	pm->led.static_brightness = brightness;
	return powermate_set_led(pm, NULL);
	}


int powermate_set_pulse_speed(PowerMate *pm, unsigned short speed)
	{
	pm->led.pulse_speed = speed;
	return powermate_set_led(pm, NULL);
	}


int powermate_set_pulse_table(PowerMate *pm, unsigned char table)
	{
	pm->led.pulse_table = table;
	return powermate_set_led(pm, NULL);
	}


int powermate_set_pulse_asleep(PowerMate *pm, unsigned char state)
	{
	pm->led.pulse_asleep = state;
	return powermate_set_led(pm, NULL);
	}


int powermate_set_pulse_awake(PowerMate *pm, unsigned char state)
	{
	pm->led.pulse_awake = state;
	return powermate_set_led(pm, NULL);
	}


int powermate_set_handlers(PowerMate *pm, PowerMateHandlers *handlers)
	{
	if (handlers == NULL)
		{
		errno = EFAULT;
		return -1;
		}

	if (handlers != &pm->handlers) memmove(&pm->handlers, handlers, sizeof(PowerMateHandlers));
	return 0;
	}









/* ------ CUT --- CUT --- CUT ------ */


void on_left(PowerMate *pm, void *data, unsigned int units)
	{
	printf("Rotated left %d units\n", units);
	}

void on_right(PowerMate *pm, void *data, unsigned int units)
	{
	printf("Rotated right %d units\n", units);
	}

void on_down(PowerMate *pm, void *data)
	{
	printf("Button pushed\n");
	}

void on_up(PowerMate *pm, void *data)
	{
	printf("Button released\n");
	pm->led.pulse_speed += 1;
	if (pm->led.pulse_speed > 510) pm->led.pulse_speed = 510;
	if (powermate_set_led(pm, &pm->led) == -1) printf("ERRNO: %d\n", errno);

				printf(	"static_brightness	= %hhu\n"
					"pulse_speed		= %hu\n"
					"pulse_mode		= %hhu\n"
					"pulse_when_power_off	= %hhu\n"
					"pulse_when_power_on	= %hhu\n",
					pm->led.static_brightness,
					pm->led.pulse_speed,
					pm->led.pulse_table,
					pm->led.pulse_asleep,
					pm->led.pulse_awake);
	}


void on_led(PowerMate *pm, void *data)
	{
	}


int main(int argc, char **argv)
	{
	PowerMate *pm;
	PowerMateLED led = {199, 255, 1, 0, 1};
	PowerMateHandlers handlers = {on_left, on_right, on_down, on_up};

	printf("%d\n", NAME_MAX);
	exit(0);

	pm = powermate_new("/dev/input/event3", &handlers);
	if (powermate_set_led(pm, &led) == -1) printf("mierda\n");

	printf("%s\n", pm->model_id);

	powermate_get_events(pm);
	printf("%s desconectado!\n", pm->model_id);
	return 0;
	}
