/*
	libpowermate v1.0
	Griffin PowerMate handling library
	Copyright(C) 2004 Manuel Sainz de Baranda Goñi <lucifer@afrodia.com>
	Copyright(C) 2004 The Free Software Foundation
	Distributed under the terms of the GNU Public License v2

	Led configuration format:

					Host specific integer value
	 __ __ __ __ __ ___ ____ _____ _____________|________________________________________________
												      \
				.. .. .. 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
					 \/ \/ \___/  \_______________________/ \_____________________/
					 |  |	 |		  |			   |
	pulse when host is power on -----+  |	 |		  |			   |
	pulse when host is sleeping --------+	 |		  |			   |
	pulse mode (0, 1 or 2) ------------------+		  |			   |
	pulse speed (0 / 510, >= 286 freezes rotation events) ----+			   |
	static brightness (0 / 255) -------------------------------------------------------+

	Comments from <Linux Kernel Sources>/drivers/usb/input/powermate.c:

	the powermate takes an operation and an argument for its pulse algorithm.
	the operation can be:
	0: divide the speed
	1: pulse at normal speed
	2: multiply the speed
	the argument only has an effect for operations 0 and 2, and ranges between
	1 (least effect) to 255 (maximum effect).
       
	thus, several states are equivalent and are coalesced into one state.

	we map this onto a range from 0 to 510, with:
	0 -- 254    -- use divide (0 = slowest)
	255         -- use normal speed
	256 -- 510  -- use multiple (510 = fastest).

	Only values of 'arg' quite close to 255 are particularly useful/spectacular.

*/


#ifndef __POWERMATE_H__
#define __POWERMATE_H__

#include <sys/stat.h>



typedef enum {
	POWERMATE_PULSE_MODE_DIVIDE,
	POWERMATE_PULSE_MODE_NORMAL,
	POWERMATE_PULSE_MODE_MULTIPLY
} PowerMatePulseMode;

typedef enum {
	POWERMATE_PULSE_STATE_ON,
	POWERMATE_PULSE_STATE_OFF
} PowerMatePulseState;

enum {
	POWERMATE_PULSE_ASLEEP,
	POWERMATE_PULSE_AWAKE
};

struct PowerMate;
struct PowerMateLED;
struct PowerMateHandlers;
typedef struct PowerMate PowerMate;
typedef struct PowerMateLED PowerMateLED;
typedef struct PowerMateHandlers PowerMateHandlers;

struct PowerMate {
	int input;
	int output;
	struct stat device_stat;
	char *model_id;
	PowerMateLED led;
	PowerMateHandlers handlers;
};

struct PowerMateLED {
	unsigned char static_brightness; /* LED brightness */
	unsigned short int pulse_speed;	 /* pulsing speed modifier (0 ... 510); 0-254 = slower, 255 = standard, 256-510 = faster. */
	unsigned char pulse_table;	 /* pulse table (0, 1, 2 valid) */
	unsigned char pulse_asleep;	 /* pulse whilst asleep? */
	unsigned char pulse_awake;	 /* pulse constantly? */
};

struct PowerMateHandlers{
	void (*on_left)(PowerMate *pm, void *data, unsigned int units);
	void (*on_right)(PowerMate *pm, void *data, unsigned int units);
	void (*on_down)(PowerMate *pm, void *data);
	void (*on_up)(PowerMate *pm, void *data);
};


#define powermate_get_state(p) p->state

char*		check_powermate_device		(int fd);
PowerMate*	powermate_new			(char *device,
						PowerMateHandlers *handlers);
void		powermate_get_events		(PowerMate *pm);
int		powermate_set_led		(PowerMate *pm,
						PowerMateLED *led);
int		powermate_set_static_brightness	(PowerMate *pm,
						unsigned char brightness);
int		powermate_set_pulse_speed	(PowerMate *pm,
						unsigned short speed);
int		powermate_set_pulse_table	(PowerMate *pm,
						unsigned char table);
int		powermate_set_pulse_asleep	(PowerMate *pm,
						unsigned char state);
int		powermate_set_pulse_awake	(PowerMate *pm,
						unsigned char state);

#endif /* __POWERMATE_H__ */
